#!/usr/bin/env python

"Login redirection."

import md5

class LoginRedirectResource:

    "A resource redirecting to a login URL."

    def __init__(self, login_url, app_url, resource, authenticator, anonymous_parameter_name=None):

        """
        Initialise the resource with a 'login_url', an 'app_url' where the 'resource' for
        the application being protected should be reachable, and an 'authenticator'.

        If the optional 'anonymous_parameter_name' is set, clients providing a parameter
        of that name in the URL will not be authenticated, but then such clients will not
        get a user identity associated with them. 
        """

        self.login_url = login_url
        self.app_url = app_url
        self.resource = resource
        self.authenticator = authenticator
        self.anonymous_parameter_name = anonymous_parameter_name

    def respond(self, trans):

        # Check for the anonymous parameter, if appropriate.

        fields = trans.get_fields_from_path()
        if self.anonymous_parameter_name is not None and fields.has_key(self.anonymous_parameter_name):
            is_anonymous = 1
        else:
            is_anonymous = 0

        # Check the authentication details with the specified authenticator.

        if is_anonymous or self.authenticator.authenticate(trans):
            self.resource.respond(trans)
        else:
            # Redirect to the login URL.

            trans.set_header_value("Location", "%s?redirect=%s%s" % (self.login_url, self.app_url, trans.get_path()))
            trans.set_response_code(307)

class LoginRedirectAuthenticator:

    """
    An authenticator which verifies the credentials provided in a special login cookie.
    """

    def __init__(self, secret_key, cookie_name=None):

        "Initialise the authenticator with a 'secret_key' and an optional 'cookie_name'."

        self.secret_key = secret_key
        self.cookie_name = cookie_name or "LoginAuthenticator"

    def authenticate(self, trans):

        "Authenticate the originator of 'trans', updating the object if successful."

        cookie = trans.get_cookie(self.cookie_name)
        if cookie is None:
            return 0

        # Test the token from the cookie against a recreated token using the
        # given information.
        # NOTE: This should be moved into a common library.

        username, code = cookie.value.split(":")
        print "*", username, code
        if code == md5.md5(username + self.secret_key).hexdigest():

            # Update the transaction with the user details.

            trans.set_user(username)
            return 1
        else:
            return 0

# vim: tabstop=4 expandtab shiftwidth=4
